/******************************************************************************/
/*                                                                            */
/*                SAS Program to Score the TIMSS Advanced 2008                */
/*                               Physics Items                                */
/*                       TIMSS Advanced 2008 User Guide                       */
/*                                                                            */
/******************************************************************************/

OPTIONS NONOTES ;

* Specify the path for the SAS data files following the format x:\xxx\ ;
* <path> must be edited ;

LIBNAME LIBDAT "<path>" ;

* Specify the list of countries to be processed ;

%LET COUNTRY = ARM IRN ITA LBN NLD NOR RUS SVN SWE ;

* List of multiple-choice items with A key ;

%LET ARIGHT = PA13001  PA13009  PA13021  PA23038  PA23039  PA23042  ;

* List of multiple-choice items with B key ;

%LET BRIGHT = PA13002  PA13011  PA13014  PA13019  PA23071  PA23148  PA23131
              PA23064  PA23056  PA23142  PA23030  PA23028  ;

* List of multiple-choice items with C key ;

%LET CRIGHT = PA13003  PA13005  PA13012  PA13013  PA13015  PA13018  PA23104
              PA23048  PA23085  PA23050  PA23058  PA23110  PA23140  PA23059
              PA23138  ;

* List of multiple-choice items with D key ;

%LET DRIGHT = PA13004  PA13006  PA13016  PA13017  PA23029  PA23113  PA23115  ;

* List of constructed-response items ;

%LET CONSTR = PA13022  PA13023  PA13024  PA13025  PA13026  PA13027A PA23146
              PA23041  PA23053  PA23119  PA23088  PA23066  PA23035  PA23012
              PA23051  PA23130  PA23086  PA23072  PA23022  PA23078  PA23128
              PA23014  PA23025  PA23034  PA23044  PA23082  PA23084  PA23137  ;

%MACRO SCOREIT (ITEM, TYPE, RIGHT, NR, NA, OM, OTHER) ;

   * Code for multiple-choice items ;

   %IF %UPCASE(&TYPE) = "MC" %THEN %DO ;
      SCORE = 0 ;
         IF &ITEM = &RIGHT THEN SCORE = 1 ;
         IF &ITEM = &NR    THEN SCORE = 0 ; * Not Reached ;
         IF &ITEM = &NA    THEN SCORE = . ; * Not Administered ;
         IF &ITEM = &OM    THEN SCORE = 0 ; * Omitted ;
         IF &ITEM = &OTHER THEN SCORE = 0 ; * Other Missing ;
      &ITEM = SCORE ;
   %END ;

   * Code for constructed-response items ;

   %IF %UPCASE(&TYPE) = "CR" %THEN %DO ;
      SCORE = 0 ;
         IF &ITEM >= 20 AND &ITEM < 30 THEN SCORE = 2 ;
         IF &ITEM >= 10 AND &ITEM < 20 THEN SCORE = 1 ;
         IF &ITEM >= 70 AND &ITEM < 80 THEN SCORE = 0 ;
         IF &ITEM = &NR    THEN SCORE = 0 ; * Not Reached ;
         IF &ITEM = &NA    THEN SCORE = . ; * Not Administered ;
         IF &ITEM = &OM    THEN SCORE = 0 ; * Omitted ;
         IF &ITEM = &OTHER THEN SCORE = 0 ; * Other Missing ;
      &ITEM = SCORE ;
   %END ;

%MEND SCOREIT ;

%MACRO DOIT ;

   * Start of country processing loop ;

   %LET I = 1 ;
   %DO %WHILE(%LENGTH(%SCAN(&COUNTRY,&I))) ;
      %LET CTRY = %SCAN(&COUNTRY,&I) ;

      * Get the student achievement data ;

      DATA LIBDAT.PSC&CTRY.M2; * Output File ;
           SET LIBDAT.PSA&CTRY.M2; * Input File  ;

           ARRAY ARIGHT &ARIGHT ;
           ARRAY BRIGHT &BRIGHT ;
           ARRAY CRIGHT &CRIGHT ;
           ARRAY DRIGHT &DRIGHT ;
           ARRAY CONSTR &CONSTR ;

           DO OVER ARIGHT ; %SCOREIT (ARIGHT, "MC", 1, .R, .A, ., .I) ; END ;
           DO OVER BRIGHT ; %SCOREIT (BRIGHT, "MC", 2, .R, .A, ., .I) ; END ;
           DO OVER CRIGHT ; %SCOREIT (CRIGHT, "MC", 3, .R, .A, ., .I) ; END ;
           DO OVER DRIGHT ; %SCOREIT (DRIGHT, "MC", 4, .R, .A, ., .I) ; END ;
           DO OVER CONSTR ; %SCOREIT (CONSTR, "CR",  , .R, .A, ., .I) ; END ;

      RUN ;

      * End of country processing loop ;

      %LET I = %EVAL(&I + 1) ;
   %END ;

%MEND DOIT ;

%DOIT ;
