*                                                                              .
*                 SAS Program to Score the TIMSS Advanced 2008                 .
*                                Physics Items                                 .
*                        TIMSS Advanced 2008 User Guide                        .
*                                                                              .

SET MPRINT=ON.

DEFINE SCOREIT (TYPE  = !CHAREND('/') /
                ITEM  = !CHAREND('/') /
                RIGHT = !CHAREND('/') /
                NR    = !CHAREND('/') /
                NA    = !CHAREND('/') /
                OM    = !CHAREND('/') /
                OTHER = !CHAREND('/') ) .

* Code for multiple-choice items .

 !IF (!UPCASE(!TYPE)=MC) !THEN
    !DO !I !IN (!ITEM) .
       RECODE !I (!RIGHT = 1      )
                 (!NR    = 0      ) /* Not Reached      */
                 (!NA    = SYSMIS ) /* Not Administered */
                 (!OM    = 0      ) /* Omitted          */
                 (!OTHER = 0      ) /* Other Missing    */
                 (ELSE   = 0      ) .
    !DOEND .
 !IFEND .

 * Code for constructed-response items .

 !IF (!UPCASE(!TYPE)=CR) !THEN
    !DO !I !IN (!ITEM).
       RECODE !I (10 THRU 19 = 1      )
                 (20 THRU 29 = 2      )
                 (30 THRU 39 = 3      )
                 (70 THRU 79 = 0      )
                 (!NR        = 0      ) /* Not Reached      */
                 (!NA        = SYSMIS ) /* Not Administered */
                 (!OM        = 0      ) /* Omitted          */
                 (!OTHER     = 0      ) /* Other Missing    */
                 (ELSE       = 0      ).
    !DOEND .
 !IFEND .

!ENDDEFINE.

DEFINE DOIT (COUNTRY = !CHAREND('/') ) .

* Specify the path for the SPSS data files following the format x:\xxx\ .
* <path> must be edited .

!LET !LIBDAT = !UNQUOTE("<path>") .

* Country loop .

!DO !CTY !IN (!COUNTRY) .

* Get the student achievement data .

ADD FILES
  / FILE = !QUOTE(!CONCAT(!LIBDAT,'PSA',!CTY,'M2.SAV')) .

* Score multiple-choice items with A key .

SCOREIT  Type  = MC      /
         Item  = PA13001  PA13009  PA13021  PA23038  PA23039  PA23042  /
         RIGHT = 1       /
         NR    = 6       /
         NA    = sysmis  /
         OM    = 9       /
         OTHER = 7       .

* Score multiple-choice items with B key .

SCOREIT  Type  = MC      /
         Item  = PA13002  PA13011  PA13014  PA13019  PA23071  PA23148  PA23131
                 PA23064  PA23056  PA23142  PA23030  PA23028  /
         RIGHT = 2       /
         NR    = 6       /
         NA    = sysmis  /
         OM    = 9       /
         OTHER = 7       .

* Score multiple-choice items with C key .

SCOREIT  Type  = MC      /
         Item  = PA13003  PA13005  PA13012  PA13013  PA13015  PA13018  PA23104
                 PA23048  PA23085  PA23050  PA23058  PA23110  PA23140  PA23059
                 PA23138  /
         RIGHT = 3       /
         NR    = 6       /
         NA    = sysmis  /
         OM    = 9       /
         OTHER = 7       .

* Score multiple-choice items with D key .

SCOREIT  Type  = MC       /
         Item  = PA13004  PA13006  PA13016  PA13017  PA23029  PA23113  PA23115  /
         RIGHT = 4       /
         NR    = 6       /
         NA    = sysmis  /
         OM    = 9       /
         OTHER = 7       .

* Score Constructed-Response Items .

SCOREIT  Type  = CR      /
         Item  = PA13022  PA13023  PA13024  PA13025  PA13026  PA13027A PA23146
                 PA23041  PA23053  PA23119  PA23088  PA23066  PA23035  PA23012
                 PA23051  PA23130  PA23086  PA23072  PA23022  PA23078  PA23128
                 PA23014  PA23025  PA23034  PA23044  PA23082  PA23084  PA23137  /
         RIGHT =  0      /
         NR    = 96      /
         NA    = sysmis  /
         OM    = 99      /
         OTHER = 90      .

EXECUTE.

* Save the student data .

SAVE OUTFILE = !QUOTE(!CONCAT(!LIBDAT,'PSC',!CTY,'M2.SAV')) .

!DOEND .

!ENDDEFINE .

* Specify the list of countries to be processed .

DOIT COUNTRY = ARM IRN ITA LBN NLD NOR RUS SVN SWE .
